function CleanUI_InitItemButtons()
    CleanUIUserData = CleanUIUserData or {};

    CleanUIUserData.Hearthstone = CleanUIUserData.Hearthstone or 6948;

    -- create new buttons

    local hearthstoneButton = CleanUI_CreateItemButton("CUI_ItemButtonHearthstone", CleanUIUserData.Hearthstone, MinimapCluster, 1);
    hearthstoneButton:SetPoint("TOPRIGHT", MinimapCluster, "BOTTOMRIGHT", -5, -18);
    hearthstoneButton:SetFrameStrata("MEDIUM");
    -- hearthstoneButton:SetScale(0.7);
end

function CleanUI_CreateItemButton(name, id, parent, index)
    local button = _G[name];
    
    if (not button) then
        button = CreateFrame("Button", name, parent, "SecureActionButtonTemplate, ActionButtonTemplate");

        button:RegisterForClicks("AnyUp", "AnyDown");

        button:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN");
        button:RegisterEvent("ACTIONBAR_UPDATE_STATE");
        button:RegisterEvent("BAG_UPDATE");
        button:RegisterEvent("ZONE_CHANGED");
        button:RegisterEvent("ZONE_CHANGED_INDOORS");
        button:RegisterEvent("ZONE_CHANGED_NEW_AREA");
        button:RegisterEvent("PLAYER_REGEN_DISABLED");
        button:RegisterEvent("PLAYER_REGEN_ENABLED");

        button:RegisterForDrag();
    end
    
    button:Show();
    
    -- Texture
    local itemTexture = GetItemIcon(id); 
    button.icon:SetTexture(itemTexture);

    -- Item data

    button.asyncItem = Item:CreateFromItemID(id);

    button.asyncItem:ContinueOnItemLoad(function()
        local itemName, itemLink = GetItemInfo(id);
    
        button:SetAttribute("type1", "item");
        button:SetAttribute("type2", "item");
        button:SetAttribute("item", itemName);
        button:SetAttribute("itemId", id);
        button:SetAttribute("itemLink", itemLink);
        button:SetAttribute("index", index);
    
        CleanUI_CheckBagItemUsable(button);
    end);

    button.cooldown = _G[button:GetName().."Cooldown"];
    button.cooldown:ClearAllPoints();
    button.cooldown:SetPoint("TOPLEFT", button, "TOPLEFT", 0, -0);
    button.cooldown:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT", -0, 0);
    
    button.cooldown:SetEdgeTexture("Interface\\Cooldown\\edge");
    button.cooldown:SetSwipeColor(0, 0, 0);
    button.cooldown:SetHideCountdownNumbers(false);
    button.cooldown.currentCooldownType = COOLDOWN_TYPE_NORMAL;

    button.HighlightTexture:ClearAllPoints();
    button.HighlightTexture:SetPoint("TOPLEFT", -1, 1);
    button.HighlightTexture:SetPoint("BOTTOMRIGHT", 3, -2);
    
    button:HookScript("OnEnter", CleanUI_ShowBagItemTooltip);
    button:HookScript("OnLeave", CleanUI_HideBagItemTooltip);
    button:HookScript("OnReceiveDrag", CleanUI_ItemButton_OnReceiveDrag);
    
    button:HookScript("OnEvent", CleanUI_ItemButton_OnEvent); 
    CleanUI_ItemButton_OnEvent(button, "BAG_UPDATE"); 

    return button;
end

function CleanUI_ShowBagItemTooltip(self)
    GameTooltip_SetDefaultAnchor(GameTooltip, self);

    local id = self:GetAttribute("itemId");

    if (id) then
        GameTooltip:SetItemByID(id);

        if (additionalInfo) then
            GameTooltip:AddLine(additionalInfo);
        end

        GameTooltip:Show();
    end
end

function CleanUI_HideBagItemTooltip()
    GameTooltip:FadeOut();
end

function CleanUI_ItemButton_OnReceiveDrag(self)
    local infoType, info1, info2 = GetCursorInfo();

    if (infoType == "item" and info1 ~= nil) then
        if (self:GetAttribute("index") == 1) then
            CleanUIUserData.Hearthstone = info1;            
            CleanUI_InitItemButtons();
        end
    end

    CleanUI_BagUpdateCooldown(self);

    ClearCursor();
end

function CleanUI_ItemButton_OnEvent(self, event, ...)
    if ( event == "ACTIONBAR_UPDATE_COOLDOWN" ) then
        CleanUI_BagUpdateCooldown(self);
    end
    if ( event == "BAG_UPDATE" or event == "ACTIONBAR_UPDATE_STATE" or event == "ZONE_CHANGED" or event == "ZONE_CHANGED_INDOORS"  or event == "ZONE_CHANGED_NEW_AREA" or event == "PLAYER_REGEN_DISABLED" or event == "PLAYER_REGEN_ENABLED" ) then
        CleanUI_CheckBagItemUsable(self);
    end
    
    if ( event == "BAG_UPDATE") then
        local itemCount = GetItemCount(self:GetAttribute("itemId"));
        local counttext = _G[self:GetName().."Count"];
        
        if (itemCount > 1) then
            counttext:SetText(""..itemCount);
        else
            counttext:SetText("");
        end
    end
    
    if ( event == "CURSOR_UPDATE") then
        CleanUI_AdditionalItemButton_UpdateAlpha(self);
    end
end

function CleanUI_BagUpdateCooldown(button)
    local cooldown = button.cooldown;
    local itemId = button:GetAttribute("itemId");

    if (itemId ~= nil) then
        local start, duration, enable = GetItemCooldown(button:GetAttribute("itemId"));
        CooldownFrame_Set(cooldown, start, duration, enable);
    end
end

function CleanUI_CheckBagItemUsable(self)
    
    if (self:GetAttribute("itemId") and not InCombatLockdown()) then
        local isUsable = IsUsableItem(self:GetAttribute("itemId"))
        local hasToy = PlayerHasToy(self:GetAttribute("itemId"))
        
        self:Show();
        if (isUsable or hasToy) then
            self.icon:SetAlpha(1.0);
        else
            self.icon:SetAlpha(0.3);
        end
    end
end

function CleanUI_AdditionalItemButton_UpdateAlpha(self)
    local itemId = self:GetAttribute("itemId");

    local infoType, info1, info2 = GetCursorInfo();
    
    if (itemId ~= nil) then
        self:SetAlpha(1.0);
    elseif (infoType == "item" and info1 ~= nil) then
        self:SetAlpha(1.0);
    else
        self:SetAlpha(0.0);
    end
end