function CleanUI_InitWorldquests()
    hooksecurefunc(WorldMap_WorldQuestPinMixin, "RefreshVisuals", CleanUI_WorldQuestPinMixin_RefreshVisuals);
end

function CleanUI_WorldQuestPinMixin_RefreshVisuals(self)
    local inset = 0;
    local border = -9;
    local epic_border = -15;
    
    if (not self.cui_texture) then
        self.cui_texture = self:CreateTexture("ARTWORK");
        self.cui_texture:SetPoint("TOPLEFT", self, "TOPLEFT", inset, -inset);
        self.cui_texture:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -inset, inset);
        self.cui_texture:SetMask("Interface\\CharacterFrame\\TempPortraitAlphaMask");
    end

    if (not self.cui_faction) then
        self.cui_faction = self:CreateTexture("ARTWORK");
        self.cui_faction:SetSize(65, 65);
        self.cui_faction:SetPoint("CENTER", self, "CENTER", 25, 25);
        self.cui_faction:Show();
        self.cui_faction:SetDrawLayer("ARTWORK", 7);
    end

    self.BountyRing:SetPoint("CENTER", self, "CENTER", 0, -2.5);
    self.BountyRing:SetSize(90, 90);
    self.BountyRing:SetDesaturated(1);
    self.BountyRing:SetVertexColor(1.0, 0.2, 0.3);

    if (not self.cui_border) then
        self.cui_border = self:CreateTexture("ARTWORK");
        self.cui_border:SetPoint("TOPLEFT", self, "TOPLEFT", border, -border);
        self.cui_border:SetPoint("BOTTOMRIGHT", self, "BOTTOMRIGHT", -border, border);
        self.cui_border:SetTexture("Interface\\CharacterFrame\\TotemBorder");
        self.cui_border:SetDrawLayer("ARTWORK", 6);
    end
    
    if (not self.cui_infotext) then
        self.cui_infotext = self:CreateFontString(nil, "ARTWORK");
        self.cui_infotext:SetFontObject("GameFontNormalHuge");
        self.cui_infotext:SetWidth(self:GetWidth());
        self.cui_infotext:SetHeight(14);
        self.cui_infotext:ClearAllPoints();
        self.cui_infotext:SetPoint("TOP", self, "BOTTOM", 0, -2);
        self.cui_infotext:SetJustifyH("CENTER");
    end
    
    if (not self.cui_questname) then
        self.cui_questname = self:CreateFontString(nil, "ARTWORK");
        self.cui_questname:SetFontObject("GameFontHighlightHuge2");
        self.cui_questname:SetHeight(14);
        self.cui_questname:ClearAllPoints();
        self.cui_questname:SetPoint("LEFT", self, "RIGHT", 10, 0);
        self.cui_questname:SetJustifyH("CENTER");
        self.cui_questname:Hide();
    end

    if (not self.cui_factionMarker) then
        self.cui_factionMarker = self:CreateTexture("ARTWORK");
        self.cui_factionMarker:SetSize(27, 27);
        self.cui_factionMarker:SetPoint("CENTER", self, "TOP", 0, 0);
        self.cui_factionMarker:SetTexture("Interface\\RaidFrame\\ReadyCheck-NotReady");
        self.cui_factionMarker:SetDrawLayer("ARTWORK", 7);
    end

    if (not self.cui_petBattleMarker) then
        self.cui_petBattleMarker = self:CreateTexture("ARTWORK");
        self.cui_petBattleMarker:SetSize(27, 27);
        self.cui_petBattleMarker:SetPoint("CENTER", self, "TOPRIGHT", -6, -6);
        self.cui_petBattleMarker:SetTexture("Interface\\Icons\\Tracking_WildPet");
        self.cui_petBattleMarker:SetDrawLayer("ARTWORK", 7);
    end
    
    self.cui_infotext:SetText("");

    if (self.questID) then
        local tagInfo = C_QuestLog.GetQuestTagInfo(self.questID);
        
        local numQuestCurrencies = GetNumQuestLogRewardCurrencies(self.questID);
        local numQuestLogRewards = GetNumQuestLogRewards(self.questID);
        local money = GetQuestLogRewardMoney(self.questID);

        if (numQuestCurrencies >= 1) then
            local name, texture, count, currencyID = GetQuestLogRewardCurrencyInfo(1, self.questID);

            self.cui_texture:SetTexture(texture);
            self.cui_texture:Show();
            self.cui_border:Show();
            self.Texture:Hide();

            local itemRarity = select(8, C_CurrencyInfo.GetCurrencyInfo(currencyID));

            if (itemRarity) then
                local rarityColor = ITEM_QUALITY_COLORS[itemRarity];
                self.cui_border:SetDesaturated(1);
                self.cui_border:SetVertexColor(rarityColor.r, rarityColor.g, rarityColor.b);
            else
                self.cui_border:SetDesaturated(nil);
                self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
            end

            if (count > 1) then
                self.cui_infotext:SetText(""..count);
                self.cui_infotext:Show();
            else
                self.cui_infotext:Hide();
            end
        elseif numQuestLogRewards >= 1 then
            local name, texture, count, quality, isUsable, itemId = GetQuestLogRewardInfo(1, self.questID);
            self.cui_texture:SetTexture(texture);
            self.cui_texture:Show();
            self.cui_border:Show();
            self.Texture:Hide();

            if (quality) then
                local r, g, b = GetItemQualityColor(quality);
                self.cui_border:SetDesaturated(1);
                self.cui_border:SetVertexColor(r, g, b);
            else
                self.cui_border:SetDesaturated(nil);
                self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
            end
        elseif money > 0 then
            self.cui_texture:SetTexture("Interface\\Icons\\inv_misc_coin_01");
            self.cui_texture:Show();
            self.cui_border:Show();
            self.cui_infotext:Hide();

            self.Texture:Hide();

            self.cui_border:SetDesaturated(nil);
            self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
        else
            self.cui_texture:Hide();
            self.cui_border:Hide();
            self.cui_infotext:Hide();
            self.Texture:Show();

            self.cui_border:SetDesaturated(nil);
            self.cui_border:SetVertexColor(1.0, 1.0, 1.0);
        end

        if ( tagInfo.worldQuestType == Enum.QuestTagType.PetBattle ) then
            self.cui_petBattleMarker:Show();
        else
            self.cui_petBattleMarker:Hide();
        end

        local questName, factionID = C_TaskQuest.GetQuestInfoByQuestID(self.questID);

        if (IsAltKeyDown()) then
            self.cui_questname:Show();
            self.cui_questname:SetText(questName);
        else
            self.cui_questname:Hide();
        end

        if (factionID) then
            local factionName, _, standingID = GetFactionInfoByID(factionID);

            if (standingID == 8) then
                self.cui_factionMarker:Show();
            else
                self.cui_factionMarker:Hide();
            end
            self.cui_factionMarker:Hide();
        end
    end


    if (self.worldQuestType == Enum.QuestTagType.Invasion or self.worldQuestType == Enum.QuestTagType.FactionAssault) then
        local factionTag = UnitFactionGroup("player");
		if factionTag == "Alliance" then
            self.cui_faction:SetTexture("Interface\\Timer\\Alliance-Logo");
		elseif factionTag == "Horde" then
            self.cui_faction:SetTexture("Interface\\Timer\\Horde-Logo");
        else -- "Neutral"
            self.cui_faction:SetTexture("Interface\\Timer\\Panda-Logo");
        end
        
        self.cui_faction:Show();
    else
        self.cui_faction:Hide();
    end
end
